//
// WtCore
//
//
// Copyright  2005, glenn andreas, all rights reserved
//


var wt_Modules = {};

function Module(moduleName, value)
{
	 // create a new module
	 wt_Modules[moduleName] = value;
}

function Import(moduleName)
{
	 return[moduleName];
}


// The basis for the prototype based inheritence sugar - proto
function Proto(obj)
{
	 var ctor = function(fields) {
	  if (fields._mixins) {
		   wtAddMixins(this, fields._mixins); // add these before we override the things
	  }
	  for (var i in fields) {
		   if (i != '_mixins') {
			this[i] = fields[i];
		   }
	  }
	 }
	 if (obj._proto) {
	  ctor.prototype = obj._proto;
	 } else {
	  // this becomes a "root"
	  obj._proto = null; // so it is at least defined
	 }
	 var retval = new ctor(obj);
	 // we can now init it - but we only init the one specific thing (and don't do inheritence)
	 // This is wrong, because the the sub is something - so we provide two inits
	 if (obj.__init) {
	  retval.__init();
	 }
	 if (retval._init) {
	  retval._init();
	 }
	 return retval;
}

function IsArray(obj)
{
	 return (obj && typeof obj == 'object' && obj.constructor == [].constructor)
}

function IsUndefined(obj)
{
     return typeof obj == 'undefined';
}

function IsNumber(obj)
{
     return typeof obj == 'number';
}

function Clone(obj)
{
	 if (IsArray(obj)) {
	  return obj.slice(0,obj.length);
	 }
	 var ctor = obj.constructor;
	 return new ctor(obj);
}

function HasSlot(obj, name)
{
	 if (obj[name] !== null && obj[name] !== 0 && obj[name] !== false && !obj[name]) {
	  // it is undefined
	  return false;
	 }
	 // but is it us, or our proto?
	 if (obj[name] === obj._proto[name]) {
	  return false; // doesn't work for simple immutables, but good enough
	 }
	 return true;
}

// to support mixin (behavioral multiple inheritence) we have
function Mixin(obj)
{
	 return obj; // we don't flatten it - that's done when it is added
}
function wtFlatten(obj)
{
	 // flatten the inheritence of the mixin
	 var retval = {}
	 while (obj) {
	  for (var i in obj) {
		   if (retval[i]) {
			// a child has already declared this, so do nothing
		   } else {
			retval[i] = obj[i];
		   }
	  }
	  // and add in the parent
	  obj = obj._proto;
	 }
	 return retval;
}

// build the proto chain for an object
function wtProtoChain(obj)
{
	var retval = [];
	while (obj) {
		retval.push(obj);
		obj = obj._proto;
	}
	return retval;
}

function wtAddMixins(obj, mixins)
{
	 for (var i=0;i<mixins.length;i++) {
	  var mixinChain = wtProtoChain(mixins[i]);
	  // we add the mixins, starting with the most abstract prototype (and call init as we go down)
	  while (mixinChain.length) {
		   var mixin = mixinChain.pop(); // get the most abstract (end of proto chain)
		   for (var j in mixin) {
			if (j == '_init') {
			 // pass on copying init - we call it explicitly
			} else {
			 // add this mixin method - note that there may be a superclass mixin here that we replace
			 // so provide a way to retain it
			 if (obj[j]) {
				  // no real clean way to keep this...yet
				  // what we want to do is
				  //		proto = obj[j];
				  //		obj[j] = copy(mixin[j]);
				  //		obj[j].proto = proto
				  // and then
				  //		this.foo.proto.apply(this);
				  obj[j] = mixin[j];
			 } else {
				  obj[j] = mixin[j];
			 }
			}
		   }
		   if (mixin._init) {
			mixin._init.call(obj); // call the init function for mixin in the scope of the object
		   }
	  }
	 }
}

