//
// WtPersistent.js
//

protoPersistent = Proto({
     _parent : null,
     instanceName : null,
     ignoreSlots : ['_parent','_proto','_init','instanceName','ignoreSlots'],
     Dirty : function() {
	  // we write explicitly anyway
     },
     ViewChangedScript : function(slotName, value) {
	  // make sure we get the same value (since this could come from a child)
	  //this[slotName] = view[slotName];
	  if (this._parent && !IsUndefined(this._parent[slotName])) {
	       // we've got a property defined in the parent, so update that as well
	       this._parent[slotName] = this[slotName];
	       this._parent.ViewChangedScript(slotName, this);
	  }
	  if (this.ignoreSlots.findFirst(slotName) === null) {
	       this.SaveValue(this[slotName], slotName);
	  }
     },
     SaveValue : function(value, name) {
	  this.PrimSaveValue(this.Marshal(value), name);
     },
     PrimSaveValue : function(value, name) {
	  if (this.instanceName) {
	       widget.setPreferenceForKey(value, name + '-' + this.instanceName);
	  } else {
	       widget.SetPreferenceForKey(value, name);
	  }
     },
     _init : function() {
	  // load the preferences
	  for (var i in this) {
	       if (this.ignoreSlots.findFirst(i) !== null) {
		    continue; // in ignored list
	       }
	       if (typeof this[i] == 'function') {
		    continue; // don't bother with a function either
	       }
	       var value;
	       if (this.instanceName) {
		    value = widget.preferenceForKey(i + '-' + this.instanceName);
	       } else {
		    value = widget.preferenceForKey(i);
	       }
	       if (IsUndefined(value) || value === null) {
		    continue;
	       }
	       var realValue = this.Unmarshal(value);
	       //alert('loaded prefs value ' + i + ' = ' + realValue);
	       this[i] = realValue;
	  }
     },
     Marshal : function(value) {
	  //alert('Marshal ' + value);
	  var rawData = []
	  function rawEncode(str) {
	       rawData.push(str);
	  }
	  function encode(x) {
	       var i;
	       if (typeof x === 'boolean') {
		    if (x === true) {
			 rawEncode('T');
		    } else if (x === false) {
			 rawEncode('F');
		    }
	       } else if (typeof x == 'undefined') {
		    rawEncode('U');
	       } else if (x === null) {
		    rawEncode('N');
	       } else if (typeof x == 'string') {
		    rawEncode('$' + x);
	       } else if (typeof x == 'number') {
		    if (isFinite(x)) {
			 rawEncode('#' + escape(x));
		    } else {
			 rawEncode('%');
		    }
	       } else if (x && typeof x == 'object' && x.constructor == Array) {
		    rawEncode('[');
		    for (i=0;i<x.length;i++) {
			 encode(x[i]);
		    }
		    rawEncode(']');
	       }
	  }
	  encode(value);
	  return rawData.join('\n');
     },
     Unmarshal : function(str) {
	  var rawData = str.split('\n');
	  if (rawData.length === 0) {
	       return null;
	  }
	  function decode() {
	       var x = rawData.shift();
	       var retval;
	       switch (x.charAt(0)) {
		    case 'T':
		    return true;
		    case 'F':
		    return false;
		    case 'U':
		    return null; // since making 'undefined' is tricky
		    case 'N':
		    return null;
		    case '$':
		    return unescape(x.slice(1,x.length));
		    case '%':
		    return 0.0 / 0.0;
		    case '#':
		    return parseFloat(x.slice(1,x.length));
		    case '[':
		    retval = new Array();
		    while (rawData.length && rawData[0] != ']') {
			 retval.push(decode());
		    }
		    return retval;
	       }
	       return null;
	  }
	  return decode();
     }
});

protoInstancePersistent = Proto({
     _proto : protoPersistent,
     instanceName : widget.identifier
});

// so typically one would do:
//   gGlobalDefaults = Proto({_proto : protoPersistent} );
// gInstanceDefaults = Proto({_proto : gGlobalDefaults, instanceName : widget.identifier} );
//
// We could try to use _parent to layer instance on global, but there is no clean way
// to "getter" the undefined values
